module CdT
	class TimeDuration
		# Creating instances

		def self.from_seconds(amount_of_seconds)
			self.new(0).tap { |duration|
				duration.set_amount_of_seconds(amount_of_seconds)
			}.freeze
		end

		def self.from(from_date_time, to:)
			from_seconds( (to - from_date_time) * Days::CONVERTION_CONSTANT)
		end

		def self.[](amount)
			self.new(amount).freeze
		end

		# Initializing

		def initialize(amount)
			@amount_of_seconds = amount * self.class::CONVERTION_CONSTANT
		end

		# Accessing

		def amount_of_seconds
			@amount_of_seconds
		end

		def units
			CdT.subclass_responsibility
		end

		# Converting

		def to_f()
			amount_of_seconds.to_f / self.class::CONVERTION_CONSTANT
		end

		def to_i()
			to_f.to_i
		end

		def as_seconds()
			Seconds.from_seconds(amount_of_seconds)
		end

		def as_minutes()
			Minutes.from_seconds(amount_of_seconds)
		end

		def as_hours()
			Hours.from_seconds(amount_of_seconds)
		end

		def as_days()
			Days.from_seconds(amount_of_seconds)
		end

		def as_weeks()
			Weeks.from_seconds(amount_of_seconds)
		end

		# Arithmetic

		def +(time_duration)
			self.class.from_seconds(amount_of_seconds + time_duration.amount_of_seconds)
		end

		def -(time_duration)
			self.class.from_seconds(amount_of_seconds - time_duration.amount_of_seconds)
		end

		def *(scalar)
			self.class.from_seconds(amount_of_seconds * scalar)
		end

		def /(scalar)
			self.class.from_seconds(amount_of_seconds.to_f / scalar)
		end

		# Comparing

		def <(time_duration)
			amount_of_seconds < time_duration.amount_of_seconds
		end

		def <=(time_duration)
			amount_of_seconds <= time_duration.amount_of_seconds
		end

		def >(time_duration)
			amount_of_seconds > time_duration.amount_of_seconds
		end

		def >=(time_duration)
			amount_of_seconds >= time_duration.amount_of_seconds
		end

		def ==(time_duration)
			amount_of_seconds == time_duration.amount_of_seconds
		end

		# Private - Set the amount of seconds
		def set_amount_of_seconds(amount_of_seconds)
			@amount_of_seconds = amount_of_seconds
		end
	end
end